package com.sun.imageio.plugins.png;

import java.util.StringTokenizer;
import javax.imageio.metadata.IIOInvalidTreeException;
import javax.imageio.metadata.IIOMetadataNode;
import org.w3c.dom.Node;
import java.util.Iterator;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import javax.imageio.ImageTypeSpecifier;
import java.util.ArrayList;
import javax.imageio.metadata.IIOMetadata;

public class PNGMetadata extends IIOMetadata implements Cloneable
{
    public static final String nativeMetadataFormatName = "javax_imageio_png_1.0";
    protected static final String nativeMetadataFormatClassName = "com.sun.imageio.plugins.png.PNGMetadataFormat";
    static final String[] IHDR_colorTypeNames;
    static final int[] IHDR_numChannels;
    static final String[] IHDR_bitDepths;
    static final String[] IHDR_compressionMethodNames;
    static final String[] IHDR_filterMethodNames;
    static final String[] IHDR_interlaceMethodNames;
    static final String[] iCCP_compressionMethodNames;
    static final String[] zTXt_compressionMethodNames;
    public static final int PHYS_UNIT_UNKNOWN = 0;
    public static final int PHYS_UNIT_METER = 1;
    static final String[] unitSpecifierNames;
    static final String[] renderingIntentNames;
    static final String[] colorSpaceTypeNames;
    public boolean IHDR_present;
    public int IHDR_width;
    public int IHDR_height;
    public int IHDR_bitDepth;
    public int IHDR_colorType;
    public int IHDR_compressionMethod;
    public int IHDR_filterMethod;
    public int IHDR_interlaceMethod;
    public boolean PLTE_present;
    public byte[] PLTE_red;
    public byte[] PLTE_green;
    public byte[] PLTE_blue;
    public int[] PLTE_order;
    public boolean bKGD_present;
    public int bKGD_colorType;
    public int bKGD_index;
    public int bKGD_gray;
    public int bKGD_red;
    public int bKGD_green;
    public int bKGD_blue;
    public boolean cHRM_present;
    public int cHRM_whitePointX;
    public int cHRM_whitePointY;
    public int cHRM_redX;
    public int cHRM_redY;
    public int cHRM_greenX;
    public int cHRM_greenY;
    public int cHRM_blueX;
    public int cHRM_blueY;
    public boolean gAMA_present;
    public int gAMA_gamma;
    public boolean hIST_present;
    public char[] hIST_histogram;
    public boolean iCCP_present;
    public String iCCP_profileName;
    public int iCCP_compressionMethod;
    public byte[] iCCP_compressedProfile;
    public ArrayList<String> iTXt_keyword;
    public ArrayList<Boolean> iTXt_compressionFlag;
    public ArrayList<Integer> iTXt_compressionMethod;
    public ArrayList<String> iTXt_languageTag;
    public ArrayList<String> iTXt_translatedKeyword;
    public ArrayList<String> iTXt_text;
    public boolean pHYs_present;
    public int pHYs_pixelsPerUnitXAxis;
    public int pHYs_pixelsPerUnitYAxis;
    public int pHYs_unitSpecifier;
    public boolean sBIT_present;
    public int sBIT_colorType;
    public int sBIT_grayBits;
    public int sBIT_redBits;
    public int sBIT_greenBits;
    public int sBIT_blueBits;
    public int sBIT_alphaBits;
    public boolean sPLT_present;
    public String sPLT_paletteName;
    public int sPLT_sampleDepth;
    public int[] sPLT_red;
    public int[] sPLT_green;
    public int[] sPLT_blue;
    public int[] sPLT_alpha;
    public int[] sPLT_frequency;
    public boolean sRGB_present;
    public int sRGB_renderingIntent;
    public ArrayList<String> tEXt_keyword;
    public ArrayList<String> tEXt_text;
    public boolean tIME_present;
    public int tIME_year;
    public int tIME_month;
    public int tIME_day;
    public int tIME_hour;
    public int tIME_minute;
    public int tIME_second;
    public boolean tRNS_present;
    public int tRNS_colorType;
    public byte[] tRNS_alpha;
    public int tRNS_gray;
    public int tRNS_red;
    public int tRNS_green;
    public int tRNS_blue;
    public ArrayList<String> zTXt_keyword;
    public ArrayList<Integer> zTXt_compressionMethod;
    public ArrayList<String> zTXt_text;
    public ArrayList<String> unknownChunkType;
    public ArrayList<byte[]> unknownChunkData;
    
    public PNGMetadata() {
        super(true, "javax_imageio_png_1.0", "com.sun.imageio.plugins.png.PNGMetadataFormat", null, (String[])null);
        this.PLTE_order = null;
        this.iTXt_keyword = new ArrayList();
        this.iTXt_compressionFlag = new ArrayList();
        this.iTXt_compressionMethod = new ArrayList();
        this.iTXt_languageTag = new ArrayList();
        this.iTXt_translatedKeyword = new ArrayList();
        this.iTXt_text = new ArrayList();
        this.tEXt_keyword = new ArrayList();
        this.tEXt_text = new ArrayList();
        this.zTXt_keyword = new ArrayList();
        this.zTXt_compressionMethod = new ArrayList();
        this.zTXt_text = new ArrayList();
        this.unknownChunkType = new ArrayList();
        this.unknownChunkData = new ArrayList();
    }
    
    public PNGMetadata(IIOMetadata iioMetadata) {
        this.PLTE_order = null;
        this.iTXt_keyword = new ArrayList();
        this.iTXt_compressionFlag = new ArrayList();
        this.iTXt_compressionMethod = new ArrayList();
        this.iTXt_languageTag = new ArrayList();
        this.iTXt_translatedKeyword = new ArrayList();
        this.iTXt_text = new ArrayList();
        this.tEXt_keyword = new ArrayList();
        this.tEXt_text = new ArrayList();
        this.zTXt_keyword = new ArrayList();
        this.zTXt_compressionMethod = new ArrayList();
        this.zTXt_text = new ArrayList();
        this.unknownChunkType = new ArrayList();
        this.unknownChunkData = new ArrayList();
    }
    
    public void initialize(ImageTypeSpecifier imageTypeSpecifier, int n) {
        ColorModel colorModel = imageTypeSpecifier.getColorModel();
        int[] sampleSize = imageTypeSpecifier.getSampleModel().getSampleSize();
        int ihdr_bitDepth = sampleSize[0];
        for (int i = 1; i < sampleSize.length; ++i) {
            if (sampleSize[i] > ihdr_bitDepth) {
                ihdr_bitDepth = sampleSize[i];
            }
        }
        if (sampleSize.length > 1 && ihdr_bitDepth < 8) {
            ihdr_bitDepth = 8;
        }
        if (ihdr_bitDepth > 2 && ihdr_bitDepth < 4) {
            ihdr_bitDepth = 4;
        }
        else if (ihdr_bitDepth > 4 && ihdr_bitDepth < 8) {
            ihdr_bitDepth = 8;
        }
        else if (ihdr_bitDepth > 8 && ihdr_bitDepth < 16) {
            ihdr_bitDepth = 16;
        }
        else if (ihdr_bitDepth > 16) {
            throw new RuntimeException("bitDepth > 16!");
        }
        this.IHDR_bitDepth = ihdr_bitDepth;
        if ((colorModel instanceof IndexColorModel)) {
            IndexColorModel indexColorModel = (IndexColorModel)colorModel;
            int mapSize = indexColorModel.getMapSize();
            byte[] r = new byte[mapSize];
            indexColorModel.getReds(r);
            byte[] g = new byte[mapSize];
            indexColorModel.getGreens(g);
            byte[] b = new byte[mapSize];
            indexColorModel.getBlues(b);
            boolean n2 = false;
            if (!this.IHDR_present || this.IHDR_colorType != 3) {
                n2 = true;
                int n3 = 255 / ((1 << this.IHDR_bitDepth) - 1);
                int j = 0;
                while (j < mapSize) {
                    int n4 = r[j];
                    if (n4 != (byte)(j * n3) || n4 != g[j] || n4 != b[j]) {
                        n2 = false;
                        break;
                    }
                    ++j;
                }
            }
            int hasAlpha = colorModel.hasAlpha();
            byte[] a = null;
            if (!(hasAlpha == 0)) {
                a = new byte[mapSize];
                indexColorModel.getAlphas(a);
            }
            if (n2 && hasAlpha && (ihdr_bitDepth == 8 || ihdr_bitDepth == 16)) {
                this.IHDR_colorType = 4;
            }
            else if (n2 && hasAlpha == 0) {
                this.IHDR_colorType = 0;
            }
            else {
                this.IHDR_colorType = 3;
                this.PLTE_present = true;
                this.PLTE_order = null;
                this.PLTE_red = (byte[])(byte[])r.clone();
                this.PLTE_green = (byte[])(byte[])g.clone();
                this.PLTE_blue = (byte[])(byte[])b.clone();
                if (!(hasAlpha == 0)) {
                    this.tRNS_present = true;
                    this.tRNS_colorType = 3;
                    this.PLTE_order = new int[a.length];
                    byte[] array = new byte[a.length];
                    int n5 = 0;
                    for (int k = 0; k < a.length; ++k) {
                        if (a[k] != -1) {
                            array[this.PLTE_order[k] = n5] = a[k];
                            ++n5;
                        }
                    }
                    for (int l = 0; l < a.length; ++l) {
                        if (a[l] == -1) {
                            this.PLTE_order[l] = n5++;
                        }
                    }
                    byte[] plte_red = this.PLTE_red;
                    byte[] plte_green = this.PLTE_green;
                    byte[] plte_blue = this.PLTE_blue;
                    int length = plte_red.length;
                    this.PLTE_red = new byte[length];
                    this.PLTE_green = new byte[length];
                    this.PLTE_blue = new byte[length];
                    for (int n6 = 0; n6 < length; ++n6) {
                        this.PLTE_red[this.PLTE_order[n6]] = plte_red[n6];
                        this.PLTE_green[this.PLTE_order[n6]] = plte_green[n6];
                        this.PLTE_blue[this.PLTE_order[n6]] = plte_blue[n6];
                    }
                    System.arraycopy(array, 0, this.tRNS_alpha = new byte[n5], 0, n5);
                }
            }
        }
        else if (n == 1) {
            this.IHDR_colorType = 0;
        }
        else if (n == 2) {
            this.IHDR_colorType = 4;
        }
        else if (n == 3) {
            this.IHDR_colorType = 2;
        }
        else if (n == 4) {
            this.IHDR_colorType = 6;
        }
        else {
            throw new RuntimeException("Number of bands not 1-4!");
        }
        this.IHDR_present = true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    private ArrayList<byte[]> cloneBytesArrayList(ArrayList<byte[]> list) {
        if (list == null) {
            return null;
        }
        ArrayList list2 = new ArrayList(list.size());
        for (byte[] array : list) {
            list2.add((array == null) ? null : ((byte[])(byte[])array.clone()));
        }
        return list2;
    }
    
    public Object clone() {
        PNGMetadata pngMetadata;
        try {
            pngMetadata = (PNGMetadata)super.clone();
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
        pngMetadata.unknownChunkData = this.cloneBytesArrayList(this.unknownChunkData);
        return pngMetadata;
    }
    
    @Override
    public Node getAsTree(String s) {
        if (s.equals("javax_imageio_png_1.0")) {
            return this.getNativeTree();
        }
        if (s.equals("javax_imageio_1.0")) {
            return this.getStandardTree();
        }
        throw new IllegalArgumentException("Not a recognized format!");
    }
    
    private Node getNativeTree() {
        Object newChild = null;
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("javax_imageio_png_1.0");
        if (this.IHDR_present) {
            IIOMetadataNode newChild2 = new IIOMetadataNode("IHDR");
            newChild2.setAttribute("width", Integer.toString(this.IHDR_width));
            newChild2.setAttribute("height", Integer.toString(this.IHDR_height));
            newChild2.setAttribute("bitDepth", Integer.toString(this.IHDR_bitDepth));
            newChild2.setAttribute("colorType", PNGMetadata.IHDR_colorTypeNames[this.IHDR_colorType]);
            newChild2.setAttribute("compressionMethod", PNGMetadata.IHDR_compressionMethodNames[this.IHDR_compressionMethod]);
            newChild2.setAttribute("filterMethod", PNGMetadata.IHDR_filterMethodNames[this.IHDR_filterMethod]);
            newChild2.setAttribute("interlaceMethod", PNGMetadata.IHDR_interlaceMethodNames[this.IHDR_interlaceMethod]);
            iioMetadataNode.appendChild(newChild2);
        }
        if (this.PLTE_present) {
            IIOMetadataNode newChild3 = new IIOMetadataNode("PLTE");
            for (int length = this.PLTE_red.length, i = 0; i < length; ++i) {
                IIOMetadataNode newChild4 = new IIOMetadataNode("PLTEEntry");
                newChild4.setAttribute("index", Integer.toString(i));
                newChild4.setAttribute("red", Integer.toString(this.PLTE_red[i] & 0xFF));
                newChild4.setAttribute("green", Integer.toString(this.PLTE_green[i] & 0xFF));
                newChild4.setAttribute("blue", Integer.toString(this.PLTE_blue[i] & 0xFF));
                newChild3.appendChild(newChild4);
            }
            iioMetadataNode.appendChild(newChild3);
        }
        if (this.bKGD_present) {
            IIOMetadataNode newChild5 = new IIOMetadataNode("bKGD");
            if (this.bKGD_colorType == 3) {
                newChild = new IIOMetadataNode("bKGD_Palette");
                newChild.setAttribute("index", Integer.toString(this.bKGD_index));
            }
            else if (this.bKGD_colorType == 0) {
                newChild = new IIOMetadataNode("bKGD_Grayscale");
                newChild.setAttribute("gray", Integer.toString(this.bKGD_gray));
            }
            else if (this.bKGD_colorType == 2) {
                newChild = new IIOMetadataNode("bKGD_RGB");
                newChild.setAttribute("red", Integer.toString(this.bKGD_red));
                newChild.setAttribute("green", Integer.toString(this.bKGD_green));
                newChild.setAttribute("blue", Integer.toString(this.bKGD_blue));
            }
            newChild5.appendChild(newChild);
            iioMetadataNode.appendChild(newChild5);
        }
        if (this.cHRM_present) {
            IIOMetadataNode newChild6 = new IIOMetadataNode("cHRM");
            newChild6.setAttribute("whitePointX", Integer.toString(this.cHRM_whitePointX));
            newChild6.setAttribute("whitePointY", Integer.toString(this.cHRM_whitePointY));
            newChild6.setAttribute("redX", Integer.toString(this.cHRM_redX));
            newChild6.setAttribute("redY", Integer.toString(this.cHRM_redY));
            newChild6.setAttribute("greenX", Integer.toString(this.cHRM_greenX));
            newChild6.setAttribute("greenY", Integer.toString(this.cHRM_greenY));
            newChild6.setAttribute("blueX", Integer.toString(this.cHRM_blueX));
            newChild6.setAttribute("blueY", Integer.toString(this.cHRM_blueY));
            iioMetadataNode.appendChild(newChild6);
        }
        if (this.gAMA_present) {
            IIOMetadataNode newChild7 = new IIOMetadataNode("gAMA");
            newChild7.setAttribute("value", Integer.toString(this.gAMA_gamma));
            iioMetadataNode.appendChild(newChild7);
        }
        if (this.hIST_present) {
            IIOMetadataNode newChild8 = new IIOMetadataNode("hIST");
            for (int j = 0; j < this.hIST_histogram.length; ++j) {
                IIOMetadataNode newChild9 = new IIOMetadataNode("hISTEntry");
                newChild9.setAttribute("index", Integer.toString(j));
                newChild9.setAttribute("value", Integer.toString((int)this.hIST_histogram[j]));
                newChild8.appendChild(newChild9);
            }
            iioMetadataNode.appendChild(newChild8);
        }
        if (this.iCCP_present) {
            IIOMetadataNode newChild10 = new IIOMetadataNode("iCCP");
            newChild10.setAttribute("profileName", this.iCCP_profileName);
            newChild10.setAttribute("compressionMethod", PNGMetadata.iCCP_compressionMethodNames[this.iCCP_compressionMethod]);
            byte[] userObject = this.iCCP_compressedProfile;
            if (userObject != null) {
                userObject = ((byte[])(byte[])userObject).clone();
            }
            newChild10.setUserObject(userObject);
            iioMetadataNode.appendChild(newChild10);
        }
        if (this.iTXt_keyword.size() > 0) {
            IIOMetadataNode newChild11 = new IIOMetadataNode("iTXt");
            for (int k = 0; k < this.iTXt_keyword.size(); ++k) {
                IIOMetadataNode newChild12 = new IIOMetadataNode("iTXtEntry");
                newChild12.setAttribute("keyword", (String)this.iTXt_keyword.get(k));
                newChild12.setAttribute("compressionFlag", ((boolean)this.iTXt_compressionFlag.get(k)) ? "TRUE" : "FALSE");
                newChild12.setAttribute("compressionMethod", ((Integer)this.iTXt_compressionMethod.get(k)).toString());
                newChild12.setAttribute("languageTag", (String)this.iTXt_languageTag.get(k));
                newChild12.setAttribute("translatedKeyword", (String)this.iTXt_translatedKeyword.get(k));
                newChild12.setAttribute("text", (String)this.iTXt_text.get(k));
                newChild11.appendChild(newChild12);
            }
            iioMetadataNode.appendChild(newChild11);
        }
        if (this.pHYs_present) {
            IIOMetadataNode newChild13 = new IIOMetadataNode("pHYs");
            newChild13.setAttribute("pixelsPerUnitXAxis", Integer.toString(this.pHYs_pixelsPerUnitXAxis));
            newChild13.setAttribute("pixelsPerUnitYAxis", Integer.toString(this.pHYs_pixelsPerUnitYAxis));
            newChild13.setAttribute("unitSpecifier", PNGMetadata.unitSpecifierNames[this.pHYs_unitSpecifier]);
            iioMetadataNode.appendChild(newChild13);
        }
        if (this.sBIT_present) {
            IIOMetadataNode newChild14 = new IIOMetadataNode("sBIT");
            if (this.sBIT_colorType == 0) {
                newChild = new IIOMetadataNode("sBIT_Grayscale");
                newChild.setAttribute("gray", Integer.toString(this.sBIT_grayBits));
            }
            else if (this.sBIT_colorType == 4) {
                newChild = new IIOMetadataNode("sBIT_GrayAlpha");
                newChild.setAttribute("gray", Integer.toString(this.sBIT_grayBits));
                newChild.setAttribute("alpha", Integer.toString(this.sBIT_alphaBits));
            }
            else if (this.sBIT_colorType == 2) {
                newChild = new IIOMetadataNode("sBIT_RGB");
                newChild.setAttribute("red", Integer.toString(this.sBIT_redBits));
                newChild.setAttribute("green", Integer.toString(this.sBIT_greenBits));
                newChild.setAttribute("blue", Integer.toString(this.sBIT_blueBits));
            }
            else if (this.sBIT_colorType == 6) {
                newChild = new IIOMetadataNode("sBIT_RGBAlpha");
                newChild.setAttribute("red", Integer.toString(this.sBIT_redBits));
                newChild.setAttribute("green", Integer.toString(this.sBIT_greenBits));
                newChild.setAttribute("blue", Integer.toString(this.sBIT_blueBits));
                newChild.setAttribute("alpha", Integer.toString(this.sBIT_alphaBits));
            }
            else if (this.sBIT_colorType == 3) {
                newChild = new IIOMetadataNode("sBIT_Palette");
                newChild.setAttribute("red", Integer.toString(this.sBIT_redBits));
                newChild.setAttribute("green", Integer.toString(this.sBIT_greenBits));
                newChild.setAttribute("blue", Integer.toString(this.sBIT_blueBits));
            }
            newChild14.appendChild(newChild);
            iioMetadataNode.appendChild(newChild14);
        }
        if (this.sPLT_present) {
            IIOMetadataNode newChild15 = new IIOMetadataNode("sPLT");
            newChild15.setAttribute("name", this.sPLT_paletteName);
            newChild15.setAttribute("sampleDepth", Integer.toString(this.sPLT_sampleDepth));
            for (int length2 = this.sPLT_red.length, l = 0; l < length2; ++l) {
                IIOMetadataNode newChild16 = new IIOMetadataNode("sPLTEntry");
                newChild16.setAttribute("index", Integer.toString(l));
                newChild16.setAttribute("red", Integer.toString(this.sPLT_red[l]));
                newChild16.setAttribute("green", Integer.toString(this.sPLT_green[l]));
                newChild16.setAttribute("blue", Integer.toString(this.sPLT_blue[l]));
                newChild16.setAttribute("alpha", Integer.toString(this.sPLT_alpha[l]));
                newChild16.setAttribute("frequency", Integer.toString(this.sPLT_frequency[l]));
                newChild15.appendChild(newChild16);
            }
            iioMetadataNode.appendChild(newChild15);
        }
        if (this.sRGB_present) {
            IIOMetadataNode newChild17 = new IIOMetadataNode("sRGB");
            newChild17.setAttribute("renderingIntent", PNGMetadata.renderingIntentNames[this.sRGB_renderingIntent]);
            iioMetadataNode.appendChild(newChild17);
        }
        if (this.tEXt_keyword.size() > 0) {
            IIOMetadataNode newChild18 = new IIOMetadataNode("tEXt");
            for (int n = 0; n < this.tEXt_keyword.size(); ++n) {
                IIOMetadataNode newChild19 = new IIOMetadataNode("tEXtEntry");
                newChild19.setAttribute("keyword", (String)this.tEXt_keyword.get(n));
                newChild19.setAttribute("value", (String)this.tEXt_text.get(n));
                newChild18.appendChild(newChild19);
            }
            iioMetadataNode.appendChild(newChild18);
        }
        if (this.tIME_present) {
            IIOMetadataNode newChild20 = new IIOMetadataNode("tIME");
            newChild20.setAttribute("year", Integer.toString(this.tIME_year));
            newChild20.setAttribute("month", Integer.toString(this.tIME_month));
            newChild20.setAttribute("day", Integer.toString(this.tIME_day));
            newChild20.setAttribute("hour", Integer.toString(this.tIME_hour));
            newChild20.setAttribute("minute", Integer.toString(this.tIME_minute));
            newChild20.setAttribute("second", Integer.toString(this.tIME_second));
            iioMetadataNode.appendChild(newChild20);
        }
        if (this.tRNS_present) {
            IIOMetadataNode newChild21 = new IIOMetadataNode("tRNS");
            if (this.tRNS_colorType == 3) {
                newChild = new IIOMetadataNode("tRNS_Palette");
                for (int m = 0; m < this.tRNS_alpha.length; ++m) {
                    IIOMetadataNode newChild22 = new IIOMetadataNode("tRNS_PaletteEntry");
                    newChild22.setAttribute("index", Integer.toString(m));
                    newChild22.setAttribute("alpha", Integer.toString(this.tRNS_alpha[m] & 0xFF));
                    newChild.appendChild(newChild22);
                }
            }
            else if (this.tRNS_colorType == 0) {
                newChild = new IIOMetadataNode("tRNS_Grayscale");
                newChild.setAttribute("gray", Integer.toString(this.tRNS_gray));
            }
            else if (this.tRNS_colorType == 2) {
                newChild = new IIOMetadataNode("tRNS_RGB");
                newChild.setAttribute("red", Integer.toString(this.tRNS_red));
                newChild.setAttribute("green", Integer.toString(this.tRNS_green));
                newChild.setAttribute("blue", Integer.toString(this.tRNS_blue));
            }
            newChild21.appendChild(newChild);
            iioMetadataNode.appendChild(newChild21);
        }
        if (this.zTXt_keyword.size() > 0) {
            IIOMetadataNode newChild23 = new IIOMetadataNode("zTXt");
            for (int index = 0; index < this.zTXt_keyword.size(); ++index) {
                IIOMetadataNode newChild24 = new IIOMetadataNode("zTXtEntry");
                newChild24.setAttribute("keyword", (String)this.zTXt_keyword.get(index));
                newChild24.setAttribute("compressionMethod", PNGMetadata.zTXt_compressionMethodNames[(int)this.zTXt_compressionMethod.get(index)]);
                newChild24.setAttribute("text", (String)this.zTXt_text.get(index));
                newChild23.appendChild(newChild24);
            }
            iioMetadataNode.appendChild(newChild23);
        }
        if (this.unknownChunkType.size() > 0) {
            IIOMetadataNode newChild25 = new IIOMetadataNode("UnknownChunks");
            for (int n2 = 0; n2 < this.unknownChunkType.size(); ++n2) {
                IIOMetadataNode newChild26 = new IIOMetadataNode("UnknownChunk");
                newChild26.setAttribute("type", (String)this.unknownChunkType.get(n2));
                newChild26.setUserObject((byte[])(byte[])this.unknownChunkData.get(n2));
                newChild25.appendChild(newChild26);
            }
            iioMetadataNode.appendChild(newChild25);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    private int getNumChannels() {
        int n = PNGMetadata.IHDR_numChannels[this.IHDR_colorType];
        if (this.IHDR_colorType == 3 && this.tRNS_present && this.tRNS_colorType == this.IHDR_colorType) {
            n = 4;
        }
        return n;
    }
    
    public IIOMetadataNode getStandardChromaNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Chroma");
        IIOMetadataNode newChild = new IIOMetadataNode("ColorSpaceType");
        newChild.setAttribute("name", PNGMetadata.colorSpaceTypeNames[this.IHDR_colorType]);
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("NumChannels");
        newChild2.setAttribute("value", Integer.toString(this.getNumChannels()));
        iioMetadataNode.appendChild(newChild2);
        if (this.gAMA_present) {
            IIOMetadataNode newChild3 = new IIOMetadataNode("Gamma");
            newChild3.setAttribute("value", Float.toString((float)this.gAMA_gamma * 1.0E-5f));
            iioMetadataNode.appendChild(newChild3);
        }
        IIOMetadataNode newChild4 = new IIOMetadataNode("BlackIsZero");
        newChild4.setAttribute("value", "TRUE");
        iioMetadataNode.appendChild(newChild4);
        if (this.PLTE_present) {
            int n = this.tRNS_present && this.tRNS_colorType == 3;
            IIOMetadataNode newChild5 = new IIOMetadataNode("Palette");
            for (int i = 0; i < this.PLTE_red.length; ++i) {
                IIOMetadataNode newChild6 = new IIOMetadataNode("PaletteEntry");
                newChild6.setAttribute("index", Integer.toString(i));
                newChild6.setAttribute("red", Integer.toString(this.PLTE_red[i] & 0xFF));
                newChild6.setAttribute("green", Integer.toString(this.PLTE_green[i] & 0xFF));
                newChild6.setAttribute("blue", Integer.toString(this.PLTE_blue[i] & 0xFF));
                if (!(n == 0)) {
                    newChild6.setAttribute("alpha", Integer.toString((i < this.tRNS_alpha.length) ? (this.tRNS_alpha[i] & 0xFF) : 255));
                }
                newChild5.appendChild(newChild6);
            }
            iioMetadataNode.appendChild(newChild5);
        }
        if (this.bKGD_present) {
            Object newChild7;
            if (this.bKGD_colorType == 3) {
                newChild7 = new IIOMetadataNode("BackgroundIndex");
                newChild7.setAttribute("value", Integer.toString(this.bKGD_index));
            }
            else {
                newChild7 = new IIOMetadataNode("BackgroundColor");
                int bkgd_red;
                int bkgd_green;
                int j;
                if (this.bKGD_colorType == 0) {
                    bkgd_green = (bkgd_red = (j = this.bKGD_gray));
                }
                else {
                    bkgd_red = this.bKGD_red;
                    bkgd_green = this.bKGD_green;
                    j = this.bKGD_blue;
                }
                newChild7.setAttribute("red", Integer.toString(bkgd_red));
                newChild7.setAttribute("green", Integer.toString(bkgd_green));
                newChild7.setAttribute("blue", Integer.toString(j));
            }
            iioMetadataNode.appendChild(newChild7);
            return iioMetadataNode;
        }
        else {
            return iioMetadataNode;
        }
    }
    
    public IIOMetadataNode getStandardCompressionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Compression");
        IIOMetadataNode newChild = new IIOMetadataNode("CompressionTypeName");
        newChild.setAttribute("value", "deflate");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("Lossless");
        newChild2.setAttribute("value", "TRUE");
        iioMetadataNode.appendChild(newChild2);
        IIOMetadataNode newChild3 = new IIOMetadataNode("NumProgressiveScans");
        newChild3.setAttribute("value", (this.IHDR_interlaceMethod == 0) ? "1" : "7");
        iioMetadataNode.appendChild(newChild3);
        return iioMetadataNode;
    }
    
    private String repeat(String s, int n) {
        if (n == 1) {
            return s;
        }
        StringBuffer sb = new StringBuffer((s.length() + 1) * n - 1);
        sb.append(s);
        for (int i = 1; i < n; ++i) {
            sb.append(" ");
            sb.append(s);
        }
        return sb.toString();
    }
    
    public IIOMetadataNode getStandardDataNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Data");
        IIOMetadataNode newChild = new IIOMetadataNode("PlanarConfiguration");
        newChild.setAttribute("value", "PixelInterleaved");
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("SampleFormat");
        newChild2.setAttribute("value", (this.IHDR_colorType == 3) ? "Index" : "UnsignedIntegral");
        iioMetadataNode.appendChild(newChild2);
        String string = Integer.toString(this.IHDR_bitDepth);
        IIOMetadataNode newChild3 = new IIOMetadataNode("BitsPerSample");
        newChild3.setAttribute("value", this.repeat(string, this.getNumChannels()));
        iioMetadataNode.appendChild(newChild3);
        if (this.sBIT_present) {
            IIOMetadataNode newChild4 = new IIOMetadataNode("SignificantBitsPerSample");
            String s;
            if (this.sBIT_colorType == 0 || this.sBIT_colorType == 4) {
                s = Integer.toString(this.sBIT_grayBits);
            }
            else {
                s = Integer.toString(this.sBIT_redBits) + " " + Integer.toString(this.sBIT_greenBits) + " " + Integer.toString(this.sBIT_blueBits);
            }
            if (this.sBIT_colorType == 4 || this.sBIT_colorType == 6) {
                s = s + " " + Integer.toString(this.sBIT_alphaBits);
            }
            newChild4.setAttribute("value", s);
            iioMetadataNode.appendChild(newChild4);
            return iioMetadataNode;
        }
        else {
            return iioMetadataNode;
        }
    }
    
    public IIOMetadataNode getStandardDimensionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Dimension");
        IIOMetadataNode newChild = new IIOMetadataNode("PixelAspectRatio");
        newChild.setAttribute("value", Float.toString(this.pHYs_present ? ((float)this.pHYs_pixelsPerUnitXAxis / (float)this.pHYs_pixelsPerUnitYAxis) : 1.0f));
        iioMetadataNode.appendChild(newChild);
        IIOMetadataNode newChild2 = new IIOMetadataNode("ImageOrientation");
        newChild2.setAttribute("value", "Normal");
        iioMetadataNode.appendChild(newChild2);
        if (this.pHYs_present && this.pHYs_unitSpecifier == 1) {
            IIOMetadataNode newChild3 = new IIOMetadataNode("HorizontalPixelSize");
            newChild3.setAttribute("value", Float.toString(1000.0f / (float)this.pHYs_pixelsPerUnitXAxis));
            iioMetadataNode.appendChild(newChild3);
            IIOMetadataNode newChild4 = new IIOMetadataNode("VerticalPixelSize");
            newChild4.setAttribute("value", Float.toString(1000.0f / (float)this.pHYs_pixelsPerUnitYAxis));
            iioMetadataNode.appendChild(newChild4);
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardDocumentNode() {
        if (!this.tIME_present) {
            return null;
        }
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Document");
        IIOMetadataNode newChild = new IIOMetadataNode("ImageModificationTime");
        newChild.setAttribute("year", Integer.toString(this.tIME_year));
        newChild.setAttribute("month", Integer.toString(this.tIME_month));
        newChild.setAttribute("day", Integer.toString(this.tIME_day));
        newChild.setAttribute("hour", Integer.toString(this.tIME_hour));
        newChild.setAttribute("minute", Integer.toString(this.tIME_minute));
        newChild.setAttribute("second", Integer.toString(this.tIME_second));
        iioMetadataNode.appendChild(newChild);
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardTextNode() {
        if (this.tEXt_keyword.size() + this.iTXt_keyword.size() + this.zTXt_keyword.size() == 0) {
            return null;
        }
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Text");
        for (int i = 0; i < this.tEXt_keyword.size(); ++i) {
            IIOMetadataNode newChild = new IIOMetadataNode("TextEntry");
            newChild.setAttribute("keyword", (String)this.tEXt_keyword.get(i));
            newChild.setAttribute("value", (String)this.tEXt_text.get(i));
            newChild.setAttribute("encoding", "ISO-8859-1");
            newChild.setAttribute("compression", "none");
            iioMetadataNode.appendChild(newChild);
        }
        for (int j = 0; j < this.iTXt_keyword.size(); ++j) {
            IIOMetadataNode newChild2 = new IIOMetadataNode("TextEntry");
            newChild2.setAttribute("keyword", (String)this.iTXt_keyword.get(j));
            newChild2.setAttribute("value", (String)this.iTXt_text.get(j));
            newChild2.setAttribute("language", (String)this.iTXt_languageTag.get(j));
            if ((boolean)this.iTXt_compressionFlag.get(j)) {
                newChild2.setAttribute("compression", "zip");
            }
            else {
                newChild2.setAttribute("compression", "none");
            }
            iioMetadataNode.appendChild(newChild2);
        }
        for (int k = 0; k < this.zTXt_keyword.size(); ++k) {
            IIOMetadataNode newChild3 = new IIOMetadataNode("TextEntry");
            newChild3.setAttribute("keyword", (String)this.zTXt_keyword.get(k));
            newChild3.setAttribute("value", (String)this.zTXt_text.get(k));
            newChild3.setAttribute("compression", "zip");
            iioMetadataNode.appendChild(newChild3);
        }
        return iioMetadataNode;
    }
    
    public IIOMetadataNode getStandardTransparencyNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Transparency");
        IIOMetadataNode newChild = new IIOMetadataNode("Alpha");
        newChild.setAttribute("value", (this.IHDR_colorType == 6 || this.IHDR_colorType == 4 || (this.IHDR_colorType == 3 && this.tRNS_present && this.tRNS_colorType == this.IHDR_colorType && this.tRNS_alpha != null)) ? "nonpremultipled" : "none");
        iioMetadataNode.appendChild(newChild);
        if (this.tRNS_present) {
            IIOMetadataNode newChild2 = new IIOMetadataNode("TransparentColor");
            if (this.tRNS_colorType == 2) {
                newChild2.setAttribute("value", Integer.toString(this.tRNS_red) + " " + Integer.toString(this.tRNS_green) + " " + Integer.toString(this.tRNS_blue));
            }
            else if (this.tRNS_colorType == 0) {
                newChild2.setAttribute("value", Integer.toString(this.tRNS_gray));
            }
            iioMetadataNode.appendChild(newChild2);
            return iioMetadataNode;
        }
        else {
            return iioMetadataNode;
        }
    }
    
    private void fatal(Node offendingNode, String message) throws IIOInvalidTreeException {
        throw new IIOInvalidTreeException(message, offendingNode);
    }
    
    private String getStringAttribute(Node node, String str, String s, boolean b) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(str);
        if (namedItem == null) {
            if (!b) {
                return s;
            }
            this.fatal(node, "Required attribute " + str + " not present!");
        }
        return namedItem.getNodeValue();
    }
    
    private int getIntAttribute(Node node, String s, int n, boolean b) throws IIOInvalidTreeException {
        String stringAttribute = this.getStringAttribute(node, s, null, b);
        if (stringAttribute == null) {
            return n;
        }
        return Integer.parseInt(stringAttribute);
    }
    
    private float getFloatAttribute(Node node, String s, float n, boolean b) throws IIOInvalidTreeException {
        String stringAttribute = this.getStringAttribute(node, s, null, b);
        if (stringAttribute == null) {
            return n;
        }
        return Float.parseFloat(stringAttribute);
    }
    
    private int getIntAttribute(Node node, String s) throws IIOInvalidTreeException {
        return this.getIntAttribute(node, s, -1, true);
    }
    
    private float getFloatAttribute(Node node, String s) throws IIOInvalidTreeException {
        return this.getFloatAttribute(node, s, -1.0f, true);
    }
    
    private boolean getBooleanAttribute(Node node, String s, boolean b, boolean b2) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(s);
        if (namedItem == null) {
            if (!b2) {
                return b;
            }
            this.fatal(node, "Required attribute " + s + " not present!");
        }
        String nodeValue = namedItem.getNodeValue();
        if (nodeValue.equals("TRUE") || nodeValue.equals("true")) {
            return true;
        }
        if (nodeValue.equals("FALSE") || nodeValue.equals("false")) {
            return false;
        }
        this.fatal(node, "Attribute " + s + " must be 'TRUE' or 'FALSE'!");
        return false;
    }
    
    private boolean getBooleanAttribute(Node node, String s) throws IIOInvalidTreeException {
        return this.getBooleanAttribute(node, s, false, true);
    }
    
    private int getEnumeratedAttribute(Node node, String s, String[] array, int n, boolean b) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(s);
        if (namedItem == null) {
            if (!b) {
                return n;
            }
            this.fatal(node, "Required attribute " + s + " not present!");
        }
        String nodeValue = namedItem.getNodeValue();
        int i = 0;
        while (i < array.length) {
            if (nodeValue.equals(array[i])) {
                return i;
            }
            ++i;
        }
        this.fatal(node, "Illegal value for attribute " + s + "!");
        return -1;
    }
    
    private int getEnumeratedAttribute(Node node, String s, String[] array) throws IIOInvalidTreeException {
        return this.getEnumeratedAttribute(node, s, array, -1, true);
    }
    
    private String getAttribute(Node node, String str, String s, boolean b) throws IIOInvalidTreeException {
        Node namedItem = node.getAttributes().getNamedItem(str);
        if (namedItem == null) {
            if (!b) {
                return s;
            }
            this.fatal(node, "Required attribute " + str + " not present!");
        }
        return namedItem.getNodeValue();
    }
    
    private String getAttribute(Node node, String s) throws IIOInvalidTreeException {
        return this.getAttribute(node, s, null, true);
    }
    
    @Override
    public void mergeTree(String s, Node node) throws IIOInvalidTreeException {
        if (s.equals("javax_imageio_png_1.0")) {
            if (node == null) {
                throw new IllegalArgumentException("root == null!");
            }
            this.mergeNativeTree(node);
        }
        else if (s.equals("javax_imageio_1.0")) {
            if (node == null) {
                throw new IllegalArgumentException("root == null!");
            }
            this.mergeStandardTree(node);
        }
        else {
            throw new IllegalArgumentException("Not a recognized format!");
        }
    }
    
    private void mergeNativeTree(Node node) throws IIOInvalidTreeException {
        if (!node.getNodeName().equals("javax_imageio_png_1.0")) {
            this.fatal(node, "Root must be javax_imageio_png_1.0");
        }
        for (Node node2 = node.getFirstChild(); node2 != null; node2 = node2.getNextSibling()) {
            String nodeName = node2.getNodeName();
            if (nodeName.equals("IHDR")) {
                this.IHDR_width = this.getIntAttribute(node2, "width");
                this.IHDR_height = this.getIntAttribute(node2, "height");
                this.IHDR_bitDepth = (int)Integer.valueOf(PNGMetadata.IHDR_bitDepths[this.getEnumeratedAttribute(node2, "bitDepth", PNGMetadata.IHDR_bitDepths)]);
                this.IHDR_colorType = this.getEnumeratedAttribute(node2, "colorType", PNGMetadata.IHDR_colorTypeNames);
                this.IHDR_compressionMethod = this.getEnumeratedAttribute(node2, "compressionMethod", PNGMetadata.IHDR_compressionMethodNames);
                this.IHDR_filterMethod = this.getEnumeratedAttribute(node2, "filterMethod", PNGMetadata.IHDR_filterMethodNames);
                this.IHDR_interlaceMethod = this.getEnumeratedAttribute(node2, "interlaceMethod", PNGMetadata.IHDR_interlaceMethodNames);
                this.IHDR_present = true;
            }
            else if (nodeName.equals("PLTE")) {
                byte[] array8 = new byte[256];
                byte[] array9 = new byte[256];
                byte[] array10 = new byte[256];
                int n7 = -1;
                Node node10 = node2.getFirstChild();
                if (node10 == null) {
                    this.fatal(node2, "Palette has no entries!");
                }
                while (node10 != null) {
                    if (!node10.getNodeName().equals("PLTEEntry")) {
                        this.fatal(node2, "Only a PLTEEntry may be a child of a PLTE!");
                    }
                    int intAttribute4 = this.getIntAttribute(node10, "index");
                    if (intAttribute4 < 0 || intAttribute4 > 255) {
                        this.fatal(node2, "Bad value for PLTEEntry attribute index!");
                    }
                    if (intAttribute4 > n7) {
                        n7 = intAttribute4;
                    }
                    array8[intAttribute4] = (byte)this.getIntAttribute(node10, "red");
                    array9[intAttribute4] = (byte)this.getIntAttribute(node10, "green");
                    array10[intAttribute4] = (byte)this.getIntAttribute(node10, "blue");
                    node10 = node10.getNextSibling();
                }
                int n8 = n7 + 1;
                this.PLTE_red = new byte[n8];
                this.PLTE_green = new byte[n8];
                this.PLTE_blue = new byte[n8];
                System.arraycopy(array8, 0, this.PLTE_red, 0, n8);
                System.arraycopy(array9, 0, this.PLTE_green, 0, n8);
                System.arraycopy(array10, 0, this.PLTE_blue, 0, n8);
                this.PLTE_present = true;
            }
            else if (nodeName.equals("bKGD")) {
                this.bKGD_present = false;
                Node firstChild3 = node2.getFirstChild();
                if (firstChild3 == null) {
                    this.fatal(node2, "bKGD node has no children!");
                }
                String nodeName4 = firstChild3.getNodeName();
                if (nodeName4.equals("bKGD_Palette")) {
                    this.bKGD_index = this.getIntAttribute(firstChild3, "index");
                    this.bKGD_colorType = 3;
                }
                else if (nodeName4.equals("bKGD_Grayscale")) {
                    this.bKGD_gray = this.getIntAttribute(firstChild3, "gray");
                    this.bKGD_colorType = 0;
                }
                else if (nodeName4.equals("bKGD_RGB")) {
                    this.bKGD_red = this.getIntAttribute(firstChild3, "red");
                    this.bKGD_green = this.getIntAttribute(firstChild3, "green");
                    this.bKGD_blue = this.getIntAttribute(firstChild3, "blue");
                    this.bKGD_colorType = 2;
                }
                else {
                    this.fatal(node2, "Bad child of a bKGD node!");
                }
                if (firstChild3.getNextSibling() != null) {
                    this.fatal(node2, "bKGD node has more than one child!");
                }
                this.bKGD_present = true;
            }
            else if (nodeName.equals("cHRM")) {
                this.cHRM_whitePointX = this.getIntAttribute(node2, "whitePointX");
                this.cHRM_whitePointY = this.getIntAttribute(node2, "whitePointY");
                this.cHRM_redX = this.getIntAttribute(node2, "redX");
                this.cHRM_redY = this.getIntAttribute(node2, "redY");
                this.cHRM_greenX = this.getIntAttribute(node2, "greenX");
                this.cHRM_greenY = this.getIntAttribute(node2, "greenY");
                this.cHRM_blueX = this.getIntAttribute(node2, "blueX");
                this.cHRM_blueY = this.getIntAttribute(node2, "blueY");
                this.cHRM_present = true;
            }
            else if (nodeName.equals("gAMA")) {
                this.gAMA_gamma = this.getIntAttribute(node2, "value");
                this.gAMA_present = true;
            }
            else if (nodeName.equals("hIST")) {
                char[] array7 = new char[256];
                int n5 = -1;
                Node node9 = node2.getFirstChild();
                if (node9 == null) {
                    this.fatal(node2, "hIST node has no children!");
                }
                while (node9 != null) {
                    if (!node9.getNodeName().equals("hISTEntry")) {
                        this.fatal(node2, "Only a hISTEntry may be a child of a hIST!");
                    }
                    int intAttribute3 = this.getIntAttribute(node9, "index");
                    if (intAttribute3 < 0 || intAttribute3 > 255) {
                        this.fatal(node2, "Bad value for histEntry attribute index!");
                    }
                    if (intAttribute3 > n5) {
                        n5 = intAttribute3;
                    }
                    array7[intAttribute3] = (char)this.getIntAttribute(node9, "value");
                    node9 = node9.getNextSibling();
                }
                int n6 = n5 + 1;
                System.arraycopy(array7, 0, this.hIST_histogram = new char[n6], 0, n6);
                this.hIST_present = true;
            }
            else if (nodeName.equals("iCCP")) {
                this.iCCP_profileName = this.getAttribute(node2, "profileName");
                this.iCCP_compressionMethod = this.getEnumeratedAttribute(node2, "compressionMethod", PNGMetadata.iCCP_compressionMethodNames);
                Object userObject2 = ((IIOMetadataNode)node2).getUserObject();
                if (userObject2 == null) {
                    this.fatal(node2, "No ICCP profile present in user object!");
                }
                if (!(userObject2 instanceof byte[])) {
                    this.fatal(node2, "User object not a byte array!");
                }
                this.iCCP_compressedProfile = (byte[])(byte[])((byte[])(byte[])userObject2).clone();
                this.iCCP_present = true;
            }
            else if (nodeName.equals("iTXt")) {
                for (Node node8 = node2.getFirstChild(); node8 != null; node8 = node8.getNextSibling()) {
                    if (!node8.getNodeName().equals("iTXtEntry")) {
                        this.fatal(node2, "Only an iTXtEntry may be a child of an iTXt!");
                    }
                    String attribute2 = this.getAttribute(node8, "keyword");
                    if (this.isValidKeyword(attribute2)) {
                        this.iTXt_keyword.add(attribute2);
                        this.iTXt_compressionFlag.add(Boolean.valueOf(this.getBooleanAttribute(node8, "compressionFlag")));
                        this.iTXt_compressionMethod.add(Integer.valueOf(this.getAttribute(node8, "compressionMethod")));
                        this.iTXt_languageTag.add(this.getAttribute(node8, "languageTag"));
                        this.iTXt_translatedKeyword.add(this.getAttribute(node8, "translatedKeyword"));
                        this.iTXt_text.add(this.getAttribute(node8, "text"));
                    }
                }
            }
            else if (nodeName.equals("pHYs")) {
                this.pHYs_pixelsPerUnitXAxis = this.getIntAttribute(node2, "pixelsPerUnitXAxis");
                this.pHYs_pixelsPerUnitYAxis = this.getIntAttribute(node2, "pixelsPerUnitYAxis");
                this.pHYs_unitSpecifier = this.getEnumeratedAttribute(node2, "unitSpecifier", PNGMetadata.unitSpecifierNames);
                this.pHYs_present = true;
            }
            else if (nodeName.equals("sBIT")) {
                this.sBIT_present = false;
                Node firstChild2 = node2.getFirstChild();
                if (firstChild2 == null) {
                    this.fatal(node2, "sBIT node has no children!");
                }
                String nodeName3 = firstChild2.getNodeName();
                if (nodeName3.equals("sBIT_Grayscale")) {
                    this.sBIT_grayBits = this.getIntAttribute(firstChild2, "gray");
                    this.sBIT_colorType = 0;
                }
                else if (nodeName3.equals("sBIT_GrayAlpha")) {
                    this.sBIT_grayBits = this.getIntAttribute(firstChild2, "gray");
                    this.sBIT_alphaBits = this.getIntAttribute(firstChild2, "alpha");
                    this.sBIT_colorType = 4;
                }
                else if (nodeName3.equals("sBIT_RGB")) {
                    this.sBIT_redBits = this.getIntAttribute(firstChild2, "red");
                    this.sBIT_greenBits = this.getIntAttribute(firstChild2, "green");
                    this.sBIT_blueBits = this.getIntAttribute(firstChild2, "blue");
                    this.sBIT_colorType = 2;
                }
                else if (nodeName3.equals("sBIT_RGBAlpha")) {
                    this.sBIT_redBits = this.getIntAttribute(firstChild2, "red");
                    this.sBIT_greenBits = this.getIntAttribute(firstChild2, "green");
                    this.sBIT_blueBits = this.getIntAttribute(firstChild2, "blue");
                    this.sBIT_alphaBits = this.getIntAttribute(firstChild2, "alpha");
                    this.sBIT_colorType = 6;
                }
                else if (nodeName3.equals("sBIT_Palette")) {
                    this.sBIT_redBits = this.getIntAttribute(firstChild2, "red");
                    this.sBIT_greenBits = this.getIntAttribute(firstChild2, "green");
                    this.sBIT_blueBits = this.getIntAttribute(firstChild2, "blue");
                    this.sBIT_colorType = 3;
                }
                else {
                    this.fatal(node2, "Bad child of an sBIT node!");
                }
                if (firstChild2.getNextSibling() != null) {
                    this.fatal(node2, "sBIT node has more than one child!");
                }
                this.sBIT_present = true;
            }
            else if (nodeName.equals("sPLT")) {
                this.sPLT_paletteName = this.getAttribute(node2, "name");
                this.sPLT_sampleDepth = this.getIntAttribute(node2, "sampleDepth");
                int[] array2 = new int[256];
                int[] array3 = new int[256];
                int[] array4 = new int[256];
                int[] array5 = new int[256];
                int[] array6 = new int[256];
                int n3 = -1;
                Node node7 = node2.getFirstChild();
                if (node7 == null) {
                    this.fatal(node2, "sPLT node has no children!");
                }
                while (node7 != null) {
                    if (!node7.getNodeName().equals("sPLTEntry")) {
                        this.fatal(node2, "Only an sPLTEntry may be a child of an sPLT!");
                    }
                    int intAttribute2 = this.getIntAttribute(node7, "index");
                    if (intAttribute2 < 0 || intAttribute2 > 255) {
                        this.fatal(node2, "Bad value for PLTEEntry attribute index!");
                    }
                    if (intAttribute2 > n3) {
                        n3 = intAttribute2;
                    }
                    array2[intAttribute2] = this.getIntAttribute(node7, "red");
                    array3[intAttribute2] = this.getIntAttribute(node7, "green");
                    array4[intAttribute2] = this.getIntAttribute(node7, "blue");
                    array5[intAttribute2] = this.getIntAttribute(node7, "alpha");
                    array6[intAttribute2] = this.getIntAttribute(node7, "frequency");
                    node7 = node7.getNextSibling();
                }
                int n4 = n3 + 1;
                this.sPLT_red = new int[n4];
                this.sPLT_green = new int[n4];
                this.sPLT_blue = new int[n4];
                this.sPLT_alpha = new int[n4];
                this.sPLT_frequency = new int[n4];
                System.arraycopy(array2, 0, this.sPLT_red, 0, n4);
                System.arraycopy(array3, 0, this.sPLT_green, 0, n4);
                System.arraycopy(array4, 0, this.sPLT_blue, 0, n4);
                System.arraycopy(array5, 0, this.sPLT_alpha, 0, n4);
                System.arraycopy(array6, 0, this.sPLT_frequency, 0, n4);
                this.sPLT_present = true;
            }
            else if (nodeName.equals("sRGB")) {
                this.sRGB_renderingIntent = this.getEnumeratedAttribute(node2, "renderingIntent", PNGMetadata.renderingIntentNames);
                this.sRGB_present = true;
            }
            else if (nodeName.equals("tEXt")) {
                for (Node node6 = node2.getFirstChild(); node6 != null; node6 = node6.getNextSibling()) {
                    if (!node6.getNodeName().equals("tEXtEntry")) {
                        this.fatal(node2, "Only an tEXtEntry may be a child of an tEXt!");
                    }
                    this.tEXt_keyword.add(this.getAttribute(node6, "keyword"));
                    this.tEXt_text.add(this.getAttribute(node6, "value"));
                }
            }
            else if (nodeName.equals("tIME")) {
                this.tIME_year = this.getIntAttribute(node2, "year");
                this.tIME_month = this.getIntAttribute(node2, "month");
                this.tIME_day = this.getIntAttribute(node2, "day");
                this.tIME_hour = this.getIntAttribute(node2, "hour");
                this.tIME_minute = this.getIntAttribute(node2, "minute");
                this.tIME_second = this.getIntAttribute(node2, "second");
                this.tIME_present = true;
            }
            else if (nodeName.equals("tRNS")) {
                this.tRNS_present = false;
                Node firstChild = node2.getFirstChild();
                if (firstChild == null) {
                    this.fatal(node2, "tRNS node has no children!");
                }
                String nodeName2 = firstChild.getNodeName();
                if (nodeName2.equals("tRNS_Palette")) {
                    byte[] array = new byte[256];
                    int n = -1;
                    Node node5 = firstChild.getFirstChild();
                    if (node5 == null) {
                        this.fatal(node2, "tRNS_Palette node has no children!");
                    }
                    while (node5 != null) {
                        if (!node5.getNodeName().equals("tRNS_PaletteEntry")) {
                            this.fatal(node2, "Only a tRNS_PaletteEntry may be a child of a tRNS_Palette!");
                        }
                        int intAttribute = this.getIntAttribute(node5, "index");
                        if (intAttribute < 0 || intAttribute > 255) {
                            this.fatal(node2, "Bad value for tRNS_PaletteEntry attribute index!");
                        }
                        if (intAttribute > n) {
                            n = intAttribute;
                        }
                        array[intAttribute] = (byte)this.getIntAttribute(node5, "alpha");
                        node5 = node5.getNextSibling();
                    }
                    int n2 = n + 1;
                    this.tRNS_alpha = new byte[n2];
                    this.tRNS_colorType = 3;
                    System.arraycopy(array, 0, this.tRNS_alpha, 0, n2);
                }
                else if (nodeName2.equals("tRNS_Grayscale")) {
                    this.tRNS_gray = this.getIntAttribute(firstChild, "gray");
                    this.tRNS_colorType = 0;
                }
                else if (nodeName2.equals("tRNS_RGB")) {
                    this.tRNS_red = this.getIntAttribute(firstChild, "red");
                    this.tRNS_green = this.getIntAttribute(firstChild, "green");
                    this.tRNS_blue = this.getIntAttribute(firstChild, "blue");
                    this.tRNS_colorType = 2;
                }
                else {
                    this.fatal(node2, "Bad child of a tRNS node!");
                }
                if (firstChild.getNextSibling() != null) {
                    this.fatal(node2, "tRNS node has more than one child!");
                }
                this.tRNS_present = true;
            }
            else if (nodeName.equals("zTXt")) {
                for (Node node4 = node2.getFirstChild(); node4 != null; node4 = node4.getNextSibling()) {
                    if (!node4.getNodeName().equals("zTXtEntry")) {
                        this.fatal(node2, "Only an zTXtEntry may be a child of an zTXt!");
                    }
                    this.zTXt_keyword.add(this.getAttribute(node4, "keyword"));
                    this.zTXt_compressionMethod.add(new Integer(this.getEnumeratedAttribute(node4, "compressionMethod", PNGMetadata.zTXt_compressionMethodNames)));
                    this.zTXt_text.add(this.getAttribute(node4, "text"));
                }
            }
            else if (nodeName.equals("UnknownChunks")) {
                for (Node node3 = node2.getFirstChild(); node3 != null; node3 = node3.getNextSibling()) {
                    if (!node3.getNodeName().equals("UnknownChunk")) {
                        this.fatal(node2, "Only an UnknownChunk may be a child of an UnknownChunks!");
                    }
                    String attribute = this.getAttribute(node3, "type");
                    Object userObject = ((IIOMetadataNode)node3).getUserObject();
                    if (attribute.length() != 4) {
                        this.fatal(node3, "Chunk type must be 4 characters!");
                    }
                    if (userObject == null) {
                        this.fatal(node3, "No chunk data present in user object!");
                    }
                    if (!(userObject instanceof byte[])) {
                        this.fatal(node3, "User object not a byte array!");
                    }
                    this.unknownChunkType.add(attribute);
                    this.unknownChunkData.add(((byte[])(byte[])userObject).clone());
                }
            }
            else {
                this.fatal(node2, "Unknown child of root node!");
            }
        }
    }
    
    private boolean isValidKeyword(String s) {
        int length = s.length();
        return length >= 1 && length < 80 && !s.startsWith(" ") && !s.endsWith(" ") && !s.contains("  ") && this.isISOLatin(s, false);
    }
    
    private boolean isISOLatin(String s, boolean b) {
        int length = s.length();
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i);
            if ((char1 < ' ' || char1 > 'ÿ' || (char1 > '~' && char1 < '¡')) && (!b || char1 != '\u0010')) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private void mergeStandardTree(Node node) throws IIOInvalidTreeException {
        if (!node.getNodeName().equals("javax_imageio_1.0")) {
            this.fatal(node, "Root must be javax_imageio_1.0");
        }
        for (Node node2 = node.getFirstChild(); node2 != null; node2 = node2.getNextSibling()) {
            String nodeName = node2.getNodeName();
            if (nodeName.equals("Chroma")) {
                for (Node node8 = node2.getFirstChild(); node8 != null; node8 = node8.getNextSibling()) {
                    String nodeName4 = node8.getNodeName();
                    if (nodeName4.equals("Gamma")) {
                        float floatAttribute4 = this.getFloatAttribute(node8, "value");
                        this.gAMA_present = true;
                        this.gAMA_gamma = (int)((double)(floatAttribute4 * 100000.0f) + 0.5);
                    }
                    else if (nodeName4.equals("Palette")) {
                        byte[] array = new byte[256];
                        byte[] array2 = new byte[256];
                        byte[] array3 = new byte[256];
                        int n4 = -1;
                        for (Node node9 = node8.getFirstChild(); node9 != null; node9 = node9.getNextSibling()) {
                            int intAttribute4 = this.getIntAttribute(node9, "index");
                            if (intAttribute4 >= 0 && intAttribute4 <= 255) {
                                array[intAttribute4] = (byte)this.getIntAttribute(node9, "red");
                                array2[intAttribute4] = (byte)this.getIntAttribute(node9, "green");
                                array3[intAttribute4] = (byte)this.getIntAttribute(node9, "blue");
                                if (intAttribute4 > n4) {
                                    n4 = intAttribute4;
                                }
                            }
                        }
                        int n5 = n4 + 1;
                        this.PLTE_red = new byte[n5];
                        this.PLTE_green = new byte[n5];
                        this.PLTE_blue = new byte[n5];
                        System.arraycopy(array, 0, this.PLTE_red, 0, n5);
                        System.arraycopy(array2, 0, this.PLTE_green, 0, n5);
                        System.arraycopy(array3, 0, this.PLTE_blue, 0, n5);
                        this.PLTE_present = true;
                    }
                    else if (nodeName4.equals("BackgroundIndex")) {
                        this.bKGD_present = true;
                        this.bKGD_colorType = 3;
                        this.bKGD_index = this.getIntAttribute(node8, "value");
                    }
                    else if (nodeName4.equals("BackgroundColor")) {
                        int intAttribute = this.getIntAttribute(node8, "red");
                        int intAttribute2 = this.getIntAttribute(node8, "green");
                        int intAttribute3 = this.getIntAttribute(node8, "blue");
                        if (intAttribute == intAttribute2 && intAttribute == intAttribute3) {
                            this.bKGD_colorType = 0;
                            this.bKGD_gray = intAttribute;
                        }
                        else {
                            this.bKGD_red = intAttribute;
                            this.bKGD_green = intAttribute2;
                            this.bKGD_blue = intAttribute3;
                        }
                        this.bKGD_present = true;
                    }
                }
            }
            else if (nodeName.equals("Compression")) {
                for (Node node7 = node2.getFirstChild(); node7 != null; node7 = node7.getNextSibling()) {
                    if (node7.getNodeName().equals("NumProgressiveScans")) {
                        this.IHDR_interlaceMethod = (this.getIntAttribute(node7, "value") > 1);
                    }
                }
            }
            else if (nodeName.equals("Data")) {
                for (Node node6 = node2.getFirstChild(); node6 != null; node6 = node6.getNextSibling()) {
                    String nodeName3 = node6.getNodeName();
                    if (nodeName3.equals("BitsPerSample")) {
                        StringTokenizer o2 = new StringTokenizer(this.getAttribute(node6, "value"));
                        int ihdr_bitDepth = -1;
                        while (o2.hasMoreTokens()) {
                            int int1 = Integer.parseInt(o2.nextToken());
                            if (int1 > ihdr_bitDepth) {
                                ihdr_bitDepth = int1;
                            }
                            else {
                                continue;
                            }
                        }
                        if (ihdr_bitDepth < 1) {
                            ihdr_bitDepth = 1;
                        }
                        if (ihdr_bitDepth == 3) {
                            ihdr_bitDepth = 4;
                        }
                        if (ihdr_bitDepth > 4 || ihdr_bitDepth < 8) {
                            ihdr_bitDepth = 8;
                        }
                        if (ihdr_bitDepth > 8) {
                            ihdr_bitDepth = 16;
                        }
                        this.IHDR_bitDepth = ihdr_bitDepth;
                    }
                    else if (nodeName3.equals("SignificantBitsPerSample")) {
                        StringTokenizer o = new StringTokenizer(this.getAttribute(node6, "value"));
                        int countTokens = o.countTokens();
                        if (countTokens == 1) {
                            this.sBIT_colorType = 0;
                            this.sBIT_grayBits = Integer.parseInt(o.nextToken());
                        }
                        else if (countTokens == 2) {
                            this.sBIT_colorType = 4;
                            this.sBIT_grayBits = Integer.parseInt(o.nextToken());
                            this.sBIT_alphaBits = Integer.parseInt(o.nextToken());
                        }
                        else if (countTokens == 3) {
                            this.sBIT_colorType = 2;
                            this.sBIT_redBits = Integer.parseInt(o.nextToken());
                            this.sBIT_greenBits = Integer.parseInt(o.nextToken());
                            this.sBIT_blueBits = Integer.parseInt(o.nextToken());
                        }
                        else if (countTokens == 4) {
                            this.sBIT_colorType = 6;
                            this.sBIT_redBits = Integer.parseInt(o.nextToken());
                            this.sBIT_greenBits = Integer.parseInt(o.nextToken());
                            this.sBIT_blueBits = Integer.parseInt(o.nextToken());
                            this.sBIT_alphaBits = Integer.parseInt(o.nextToken());
                        }
                        if (countTokens >= 1 && countTokens <= 4) {
                            this.sBIT_present = true;
                        }
                    }
                }
            }
            else if (nodeName.equals("Dimension")) {
                boolean n = false;
                boolean n2 = false;
                boolean n3 = false;
                float floatAttribute = -1.0f;
                float floatAttribute2 = -1.0f;
                float floatAttribute3 = -1.0f;
                for (Node node5 = node2.getFirstChild(); node5 != null; node5 = node5.getNextSibling()) {
                    String nodeName2 = node5.getNodeName();
                    if (nodeName2.equals("PixelAspectRatio")) {
                        floatAttribute3 = this.getFloatAttribute(node5, "value");
                        n3 = true;
                    }
                    else if (nodeName2.equals("HorizontalPixelSize")) {
                        floatAttribute = this.getFloatAttribute(node5, "value");
                        n = true;
                    }
                    else if (nodeName2.equals("VerticalPixelSize")) {
                        floatAttribute2 = this.getFloatAttribute(node5, "value");
                        n2 = true;
                    }
                }
                if (n && n2) {
                    this.pHYs_present = true;
                    this.pHYs_unitSpecifier = 1;
                    this.pHYs_pixelsPerUnitXAxis = (int)(floatAttribute * 1000.0f + 0.5f);
                    this.pHYs_pixelsPerUnitYAxis = (int)(floatAttribute2 * 1000.0f + 0.5f);
                }
                else if (!(n3 == 0)) {
                    this.pHYs_present = true;
                    this.pHYs_unitSpecifier = 0;
                    int phYs_pixelsPerUnitYAxis;
                    for (phYs_pixelsPerUnitYAxis = 1; phYs_pixelsPerUnitYAxis < 100 && (double)Math.abs((float)((int)(floatAttribute3 * (float)phYs_pixelsPerUnitYAxis) / phYs_pixelsPerUnitYAxis) - floatAttribute3) >= 0.001; ++phYs_pixelsPerUnitYAxis) {}
                    this.pHYs_pixelsPerUnitXAxis = (int)(floatAttribute3 * (float)phYs_pixelsPerUnitYAxis);
                    this.pHYs_pixelsPerUnitYAxis = phYs_pixelsPerUnitYAxis;
                }
            }
            else if (nodeName.equals("Document")) {
                for (Node node4 = node2.getFirstChild(); node4 != null; node4 = node4.getNextSibling()) {
                    if (node4.getNodeName().equals("ImageModificationTime")) {
                        this.tIME_present = true;
                        this.tIME_year = this.getIntAttribute(node4, "year");
                        this.tIME_month = this.getIntAttribute(node4, "month");
                        this.tIME_day = this.getIntAttribute(node4, "day");
                        this.tIME_hour = this.getIntAttribute(node4, "hour", 0, false);
                        this.tIME_minute = this.getIntAttribute(node4, "minute", 0, false);
                        this.tIME_second = this.getIntAttribute(node4, "second", 0, false);
                    }
                }
            }
            else if (nodeName.equals("Text")) {
                for (Node node3 = node2.getFirstChild(); node3 != null; node3 = node3.getNextSibling()) {
                    if (node3.getNodeName().equals("TextEntry")) {
                        String attribute = this.getAttribute(node3, "keyword", "", false);
                        String attribute2 = this.getAttribute(node3, "value");
                        String attribute3 = this.getAttribute(node3, "language", "", false);
                        String attribute4 = this.getAttribute(node3, "compression", "none", false);
                        if (this.isValidKeyword(attribute)) {
                            if (this.isISOLatin(attribute2, true)) {
                                if (attribute4.equals("zip")) {
                                    this.zTXt_keyword.add(attribute);
                                    this.zTXt_text.add(attribute2);
                                    this.zTXt_compressionMethod.add(Integer.valueOf(0));
                                }
                                else {
                                    this.tEXt_keyword.add(attribute);
                                    this.tEXt_text.add(attribute2);
                                }
                            }
                            else {
                                this.iTXt_keyword.add(attribute);
                                this.iTXt_compressionFlag.add(Boolean.valueOf(attribute4.equals("zip")));
                                this.iTXt_compressionMethod.add(Integer.valueOf(0));
                                this.iTXt_languageTag.add(attribute3);
                                this.iTXt_translatedKeyword.add(attribute);
                                this.iTXt_text.add(attribute2);
                            }
                        }
                    }
                }
            }
        }
    }
    
    @Override
    public void reset() {
        this.IHDR_present = false;
        this.PLTE_present = false;
        this.bKGD_present = false;
        this.cHRM_present = false;
        this.gAMA_present = false;
        this.hIST_present = false;
        this.iCCP_present = false;
        this.iTXt_keyword = new ArrayList();
        this.iTXt_compressionFlag = new ArrayList();
        this.iTXt_compressionMethod = new ArrayList();
        this.iTXt_languageTag = new ArrayList();
        this.iTXt_translatedKeyword = new ArrayList();
        this.iTXt_text = new ArrayList();
        this.pHYs_present = false;
        this.sBIT_present = false;
        this.sPLT_present = false;
        this.sRGB_present = false;
        this.tEXt_keyword = new ArrayList();
        this.tEXt_text = new ArrayList();
        this.tIME_present = false;
        this.tRNS_present = false;
        this.zTXt_keyword = new ArrayList();
        this.zTXt_compressionMethod = new ArrayList();
        this.zTXt_text = new ArrayList();
        this.unknownChunkType = new ArrayList();
        this.unknownChunkData = new ArrayList();
    }
    
    static {
        IHDR_colorTypeNames = new String[] { "Grayscale", null, "RGB", "Palette", "GrayAlpha", null, "RGBAlpha" };
        IHDR_numChannels = new int[] { 1, 0, 3, 3, 2, 0, 4 };
        IHDR_bitDepths = new String[] { "1", "2", "4", "8", "16" };
        IHDR_compressionMethodNames = new String[] { "deflate" };
        IHDR_filterMethodNames = new String[] { "adaptive" };
        IHDR_interlaceMethodNames = new String[] { "none", "adam7" };
        iCCP_compressionMethodNames = new String[] { "deflate" };
        zTXt_compressionMethodNames = new String[] { "deflate" };
        unitSpecifierNames = new String[] { "unknown", "meter" };
        renderingIntentNames = new String[] { "Perceptual", "Relative colorimetric", "Saturation", "Absolute colorimetric" };
        colorSpaceTypeNames = new String[] { "GRAY", null, "RGB", "RGB", "GRAY", null, "RGB" };
    }
}
